{% load static reactor %}
<link rel="stylesheet" href="{% static "reactor.css" %}">
<script defer src="{% static "morphdom.min.js" %}"></script>
<script defer src="{% static "reactor.js" %}"></script>
